/**
 * The module *shopUiLanguageswitch* registers a click on the initial element which opens a dialog containing a language switch.
 * If a language is chosen the page reloads with the new langauage depending content.
 *
 * The `de_epages.shopUiLanguageswitch` method creates dialog containing a list to switch the language.
 *
 * ### Examples
 * Creates a languageswitch dialog on the language button in the top bar.
 *
 * JavaScript:
 *
 *     $('.ep-top-bar-language-select-languageswitch').shopUiLanguageswitch({
 *         languages: [{
 *             "code": "ger",
 *             "value": "1",
 *             "translation": "Deutsch",
 *             "selected": true
 *         }, {
 *             "code": "eng",
 *             "value": "2",
 *             "translation": "Englisch",
 *             "selected": false
 *         }],
 *         location: '#IF(#Pager)#Pager.URLNoAltLanguageID#ELSE?ObjectID=#ObjectID&ViewAction=#INPUT.ViewAction[js]#ENDIF&AltLanguageID=',
 *         ofElement: languageSwitch.closest('.ep-top-bar'),
 *         position: {
 *             my: "left top",
 *             at: "left bottom+3"
 *         },
 *         translations: {
 *             tInfoHeader: 'headline',
 *             tInfoDescription: 'description'
 *         }
 *     })
 *
 * HTML:
 *
 *     <div class="ep-top-bar">
 *       <div class="ep-top-bar-language-select">
 *         <div class="ep-top-bar-button ep-top-bar-language-select-languageswitch">
 *           <span class="ep-info-bubble-globe"></span>
 *         </div>
 *       </div>
 *     </div>
 *
 *
 * @class jQuery.ui.shopUiLanguageswitch
 * @extends jQuery.widget
 *
 * @uses jquery/ui/widget
 * @uses ep/ui/dialog
 * @uses de_epages
 * @since 7.33.0
 */

/**
 * Open dialog with language list
 *
 * @method open
 * @member jQuery.ui.shopUiLanguageswitch
 *
 * @since 7.33.0
 */

/**
 * Close dialog
 *
 * @method close
 * @member jQuery.ui.shopUiLanguageswitch
 *
 * @since 7.33.0
 */

/**
 * Destroy the languageswitch widget.
 *
 * @method destroy
 * @member jQuery.ui.shopUiLanguageswitch
 *
 * @since 7.33.0
 */

/**
 * @cfg {Array} languages Array of objects containing the properties code, value, translation, selected.
 */

/**
 * @cfg {String} location Link for the page reload.
 */

/**
 * @cfg {String} position my and at part of a jQueryUI position object (see here: http://jqueryui.com/position/).
 */

/**
 * @cfg {Object} ofElement jQuery reference of a DOM element. This element is the orientation object for the position object.
 */

/**
 * @cfg {String} className Additional css class names for the dialog.
 */

/**
 * @cfg {Object} translations A map of translations for the languageswitch template.
 */

/**
 * See `jQuery.ui.shopUiLanguageswitch for details.
 *
 * @param {Object} options A map of additional options pass to the method.
 * @param {Array} languages Array of objects containing the properties code, value, translation, selected.
 * @param {Array} location Link for the page reload.
 * @param {Array} location my and at part of a jQueryUI position object (see here: http://jqueryui.com/position/).
 * @param {Array} position jQuery reference of a DOM element. This element is the orientation object for the position object.
 * @param {String} className Additional css class names for the dialog.
 * @param {Object} translations A map of translations for the languageswitch template.
 *
 * @since 7.33.0
 */

/*
 * @copyright       © Copyright 2006-2019, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages.shop.ui.languageswitch
 *
 * @revision        $Revision: 1.20 $
 */
/*globals define*/
/*jslint nomen: true*/

define('de_epages/shop/ui/languageswitch', [
    'jquery/ui/widget',
    'ep/ui/dialog',
    'de_epages',
    '$tmpl!de_epages/shop/ui/languageswitch'
], function ($, ep, de_epages, tmplLanguageswitch) {

    $.widget('ui.shopUiLanguageswitch', {
        tmplElements: {},

        options: {
        //  ofElement,
        //  topBarController,
            languages: [],
            location: '',
            position: {
                my: "left top",
                at: "left bottom+20"
            },
            className: {
                languageswitchButton: {
                    active: 'ep-top-bar-language-select-languageswitch-active'
                },
                languageswitchListItem: {
                    active: 'ep-languageswitch-list-item-active'
                }
            },
            translations: {
                tInfoHeader: 'InfoHeader',
                tInfoDescription: 'InfoDescription'
            }
        },

        _init: function() {
            var self = this,
                o = self.options,
                elem = self.element;

            // if languages for the list exist
            if (o.languages.length) {
                // if a topbar controller exists
                if (o.topBarController) {
                    // register languageswitch to controller and show topbar
                    o.topBarController.add({name: self.eventNamespace});
                    o.topBarController.show();
                }
                // register click event to open the dialog
                elem.on('click.languageSwitch', function() {
                    self.open();
                });
            }
        },


        /**
         * Method creates a dialog that contains the language list.
         *
         * @member jQuery.ui.shopUiLanguageswitch
         *
         * @since 7.33.0
         */
        _createDialog: function() {
            var self = this,
                o = self.options,
                elem = self.element;

            // if template is not already rendered
            if (!self.tmpl) {
                // render template
                self.tmpl = tmplLanguageswitch([{
                    languages: o.languages,
                    tInfoHeader: o.translations.tInfoHeader,
                    tInfoDescription: o.translations.tInfoDescription
                }]);
                // get named elements from template
                $.extend(self.tmplElements, $.tmplItem(self.tmpl).elements);
            }

            // register mouseenter/-leave events on language list to remove/add active item class
            self.tmplElements.languageList.on('mouseenter.languageswitchHover', function() {
                self.tmplElements.activeItem && self.tmplElements.activeItem.removeClass(o.className.languageswitchListItem.active);
            });
            self.tmplElements.languageList.on('mouseleave.languageswitchHover', function() {
                self.tmplElements.activeItem && self.tmplElements.activeItem.addClass(o.className.languageswitchListItem.active);
            });

            // register mouse click event on language list item to switch to the new language
            self.tmplElements.languageList.on('click.languageswitchItemClick', 'li', function() {
                var $item = $(this),
                    languageData = $item.data('language');

                // if clicked language is not the selected one => reload page with new language
                if (!languageData.selected) {
                    window.location.href = o.location + languageData.value;
                    self.close();
                }

            });

            // initialize dialog widget
            self.dialog = $('<div/>').html(self.tmpl).uiDialog({
                modal: true,
                preventScroll: true,
                closeOnClickOut: true,
                classes: {
                  'ui-dialog': 'ui-dialog-headless ui-dialog-headless-arrow'
                },
                position: self._getPosition(o.ofElement ? o.ofElement : elem),
                close: function() {
                    // remove active class from languageswitch button
                    elem.removeClass(o.className.languageswitchButton.active);
                }
            });

            // get instance of dialog widget
            self.dialogWidget = self.dialog.data('uiUiDialog');
        },


        /**
         * Prepare the language list inside the dialog (add events and so on),
         * handles clicks outside the dialog,
         * add classes.
         *
         * @member jQuery.ui.shopUiLanguageswitch
         *
         * @since 7.33.0
         */
        _prepareEnvironment: function() {
            var self = this,
                o = self.options,
                elem = self.element;

            // add active class to languageswtich button
            elem.addClass(o.className.languageswitchButton.active);

            // add active class to active list item
            self.tmplElements.activeItem.addClass(o.className.languageswitchListItem.active);
        },


        /**
         * Method returns a positioning object.
         *
         * @member jQuery.ui.shopUiLanguageswitch
         *
         * @since 7.33.0
         */
        _getPosition: function(ofElement) {
            return $.extend(this.options.position, {
                of: ofElement
            });
        },


        /**
         * Open dialog with language list
         *
         * @method open
         * @member jQuery.ui.shopUiLanguageswitch
         *
         * @since 7.33.0
         */
        open: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('open');
            } else {
                self._createDialog();
            }

            self._prepareEnvironment();
        },


        /**
         * Close dialog
         *
         * @method close
         * @member jQuery.ui.shopUiLanguageswitch
         *
         * @since 7.33.0
         */
        close: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('close');
            }
        },

        destroy: function () {
            var self = this,
                o = self.options;

            self._superApply(arguments);

            // if dialog was already rendered and a language list exists
            if (self.tmplElements.languageList) {
                // unregister mouseenter/-leave events on language list to remove/add active item class
                self.tmplElements.languageList.off('mouseenter.languageswitchHover');
                self.tmplElements.languageList.off('mouseleave.languageswitchHover');
                // unregister click event for language items
                self.tmplElements.languageList.off('click.languageswitchItemClick');
            }

            // if a topbar controller exists
            if (o.topBarController) {
                // remove languageswitch from controller
                o.topBarController.remove({name: self.eventNamespace});
            }

            // destroy dialog
            self.dialog && self.dialog.uiDialog('destroy');
        }

    });

    return de_epages;
});